package app;

import sassy.*;
import sassy.agent.*;
import java.awt.Color;
import java.io.Serializable;

/**
 *
 * @author Cole Sherer <koleslaw54@gmail.com>
 */
public class AgentBall extends Agent implements Serializable 
{

	final double SPEED = 5.0;
	final double MIN_X = 0.0;
	final double MIN_Y = 0.0;
	final double MAX_X = 800.0;
	final double MAX_Y = 600.0;

	public AgentBall() {
		super();
	}

	@Override
	public void init() {
		action = null;
	}

	@Override
	public void readConfig(Serializable config) {
		String[] s = ((String)config).split(" ");
		double radius = Double.parseDouble(s[0]);
		double x = Double.parseDouble(s[1]);
		double y = Double.parseDouble(s[2]);
		double theta = Double.parseDouble(s[3]);

		body = new Body2D(x, y, theta);
		body.addSegment(new CircleSegment(0.0, 0.0, radius));
		sensors.add(new Sensor2D(0.0, 0.0, radius, 0.0, Math.PI * 2));
	}

	@Override
	public void rcvMessage(Object content, String senderId) {}

	@Override
	public void think() {

		((Body2D)body).move(SPEED);

		final double x = ((Body2D)body).getX();
		final double y = ((Body2D)body).getY();
		double theta = ((Body2D)body).getTheta();
		final double radius = ((Body2D)body).getRadius();

		if(x + radius > MAX_X) {
			((Body2D)body).setX(MAX_X - radius);

			double vX = Math.cos(theta) * SPEED;
			double vY = Math.sin(theta) * SPEED;
			theta = Math.atan2(vY, -vX);
			((Body2D)body).setTheta(theta);
		}
		else if(x - radius < MIN_X) {
			((Body2D)body).setX(MIN_X + radius);

			double vX = Math.cos(theta) * SPEED;
			double vY = Math.sin(theta) * SPEED;
			theta = Math.atan2(vY, -vX);
			((Body2D)body).setTheta(theta);
		}
		if(y + radius > MAX_Y) {
			((Body2D)body).setY(MAX_Y - radius);

			double vX = Math.cos(theta) * SPEED;
			double vY = Math.sin(theta) * SPEED;
			theta = Math.atan2(-vY, vX);
			((Body2D)body).setTheta(theta);
		}
		else if(y - radius < MIN_Y) {
			((Body2D)body).setY(MIN_Y + radius);

			double vX = Math.cos(theta) * SPEED;
			double vY = Math.sin(theta) * SPEED;
			theta = Math.atan2(-vY, vX);
			((Body2D)body).setTheta(theta);
		}

		
		try {
			Thread.sleep((long) 1);
		}
		catch (InterruptedException e) {}

		sendMessage(new AgentBallMsg(x, y, radius), senseThinkActDelay, "gui");
	}
	

	/**
	 * Save the state of this AgentBall.
	 */
	@Override
	public Serializable saveHeadState() 
	{
		//There is no head state! The balls have no memory.
		return null;
	}


	/**
	 * Restore the state of this Agent from a saved state
	 *
	 * @param lp the saved agent state
	 */
	@Override
	public void restoreHeadState(Serializable savedHead) 
	{
		//Balls have no memory; there is no head state
		assert (savedHead == null);
	}	
}