package app;

import java.io.*;
import java.util.*;

import sassy.Creator;
import sassy.LPConfigInfo;
import sassy.PEInfo;
import sassy.Peer;

/**
 *
 * @author Cole Sherer <koleslaw54@gmail.com>
 */
public class AgentBallCreator implements Creator {
	
	/**the class name for breaking simultaneous events tie*/
	public String simulEventComparator = null;

	/**the vector for the configuration information of all the LPs*/
	List<LPConfigInfo> lpConfigInfoVector = new ArrayList<LPConfigInfo>();

	/**Distributes the LP among the available PEs
	  *@param numPEs int the number of PEs in this simulation
	  *@param numLPs int the number of LPs in this simulation
	  */
	@Override
	public List<PEInfo> appSetPEtable(int numPEs, List<LPConfigInfo> lpConfigInfoVector)
	{
		int numLPs = lpConfigInfoVector.size();
		System.out.println("inside ballCreator creator, LPs:"+ numLPs);
		for (int i = 0;i < lpConfigInfoVector.size();i++)
		{
			LPConfigInfo lpc = lpConfigInfoVector.get(i);
//			System.out.println("LP: " + lpc.LPid);
		}

		List<PEInfo> peTable = new ArrayList<PEInfo>();

		int numIMLP = 0;
		int numALP = 0;
		for(int i=0;i<lpConfigInfoVector.size();i++){
			if(lpConfigInfoVector.get(i).getLPClassName().equalsIgnoreCase("im.InterestManager")){
				numIMLP++;
			}
			if(lpConfigInfoVector.get(i).getLPClassName().equalsIgnoreCase("im.Ball")){
				numALP++;
			}
		}

		int rmderIMLP = numIMLP % numPEs;
		int rmderALP = numALP % numPEs;
		int basicQuotaIMLP = numIMLP / numPEs;
		int basicQuotaALP = numALP / numPEs;
		String pip = null;
		String pm = null;
		Peer peerStub = null;
		int begin = 0;
		int end = 0;
		int lpForEach = 0;

		//allocate IMLPs
		for (int i = 0;i < numPEs;i++)
		{
			lpForEach = basicQuotaIMLP;

			if (i < rmderIMLP)
				lpForEach++;

			//Calculate the range of LP's which will be assigned to this PE
//			begin = i * lpForEach;
			begin = end;
			end = begin + lpForEach;
//			System.out.println("PE " + i + " LPs from " + begin + " to " + end);

			PEInfo peinfo = new PEInfo(i, pip, pm, peerStub);

			//im
			int j=begin;

			for (j = begin;j < end;j++)
			{
				LPConfigInfo lpc = lpConfigInfoVector.get(j);
				System.out.println("PE " + i + " take LP " + lpc.getAPPname());//
				peinfo.lpcVector.add(lpc);
			} //end for j

			peTable.add(peinfo);
		} //end for


		//Allocate Agent LPs
		for (int i = 0;i < numPEs;i++)
		{
			lpForEach = basicQuotaALP;

			if (i < rmderALP)
				lpForEach++;

			//Calculate the range of LP's which will be assigned to this PE
			begin = end;
			end = begin + lpForEach;
//			System.out.println("PE " + i + " LPs from " + begin + " to " + end);

			PEInfo peinfo = new PEInfo(i, pip, pm, peerStub);

			int j=begin;

			for (j = begin;j < end;j++)
			{
				LPConfigInfo lpc = lpConfigInfoVector.get(j);
				System.out.println("PE " + i + " take LP " + lpc.getAPPname());//
//				peinfo.lpcVector.add(lpc);

				peTable.get(i).lpcVector.add(lpc);

			} //end for j

			System.out.println("i="+i+" ; j="+ j);
			if(i== numPEs-1 && j<lpConfigInfoVector.size()){
				for(int x=j;x<lpConfigInfoVector.size();x++){
					LPConfigInfo lpc = lpConfigInfoVector.get(x);
					System.out.println("***********PE " + i + " take LP " + lpc.getAPPname());//
//					peinfo.lpcVector.add(lpc);
					peTable.get(i).lpcVector.add(lpc);
				}
			}
//			peTable.get(i).lpcVector.add(lpc);
//			peTable.add(peinfo);
		} //end for


		return peTable;
	} //end appAllocateLPs

	
	/**Processes LP configuration information.
	  *@param fileName String the name of the configuration file
	  *@return a vector of LPConfigInfo object, each being the configuration information for an LP
	  */
	@Override
	public List<LPConfigInfo> appCreateLPs(String fileName)
	{
		//Read configuration file to decide how many instance for each class need to be created
		try
		{
			String newLine = null;
			BufferedReader bf = new BufferedReader(new FileReader(fileName));

			int numLPs = 0;
			//Each line is of the format: "<line type>:type-specific info"
			//LP's are specified as "LP:<id> <name> <class name> <application data>"
			while ((newLine = bf.readLine()) != null)
			{
				//If the first character is a '#', this is a comment. Also, ignore blank lines
				if (newLine.trim().length() == 0 || newLine.charAt(0) == '#')
					continue;

				//Extract the type of line
				int pos = newLine.indexOf(':');
				if (pos == -1)
				{
					System.err.println("Error1: Malformated configration file.");
					System.exit(1);
				}
				String lineType = newLine.substring(0, pos).trim();
				newLine = newLine.substring(pos + 1).trim();

				//Now, process the line depending on what type it is
				if (lineType.equalsIgnoreCase("LP"))
				{

					//Only split on the first 3 fields because the application data can have
					String[] tokens = newLine.split("\\s+", 4);

					if (tokens.length < 3)
					{
						System.err.println("Error: Malformated configration file.");
						System.exit(1);
					}

					String id = tokens[0].trim();
					String name = tokens[1].trim();
					String className = tokens[2].trim();
					String dat = tokens.length > 3 ? tokens[3] : "";

					//initiate interest manager LPs: Tianhao
					if(newLine.contains("InterestManager")){
						String[] s=dat.split(" ");
						int numIMLPs = Integer.parseInt(s[0])*Integer.parseInt(s[0]);
						System.out.println("Number of IMLPs "+numIMLPs);
						for(int index = 0;index<numIMLPs;index++){
							LPConfigInfo lpcInfo = new LPConfigInfo(numLPs, ((Integer)index).toString(), name+index, className, dat);
							lpConfigInfoVector.add(lpcInfo);
							numLPs++;
						}
					}

					else{
						LPConfigInfo lpcInfo = new LPConfigInfo(numLPs, id, name, className, dat);
						lpConfigInfoVector.add(lpcInfo);
						numLPs++;
					}
				}//if LP
				else if (lineType.equalsIgnoreCase("SimultenousMessageComparator"))
				{
					System.out.println("+++++++++++++");
					simulEventComparator = newLine;
					System.out.println("SimulEventComparator is "+simulEventComparator);
//					lpConfigInfoVector.add(simulEventComparator);
				}
			} //end while

			bf.close();
		}
		catch (IOException e)
		{
			System.err.println("Error: Could not read configuration file:\n\t" + e.getMessage());
		}
		System.out.println("*****Size of the LPs:"+lpConfigInfoVector.size());
		return lpConfigInfoVector;
	} //end appCreateLPs


	@Override
	public String getTieBreakerClass()
	{
		return simulEventComparator;
	}
}
