package app;

import java.io.Serializable;
import java.util.Map;
import sassy.*;
import sassy.agent.*;

/**
 *
 * @author Cole Sherer <koleslaw54@gmail.com>
 */
public class FallingBall extends Agent implements Serializable
{

	// The minimum and maximum Y values that this agent can be located at
	final double MIN_Y = 0.0;
	final double MAX_Y = 600.0;

	// The speed the agent will bounce up and down
	double speed = 0.0;

	public FallingBall() {
		super();
	}

	
	@Override
	public void init() {
		action = null;
	}


	@Override
	public void readConfig(Serializable config) {
		String[] s = ((String)config).split(" ");
		double radius = Double.parseDouble(s[0]);
		double x = Double.parseDouble(s[1]);
		double y = Double.parseDouble(s[2]);
		speed = Double.parseDouble(s[3]);

		// This test shows what will happen if you tell an agent to move
		// faster than it's body can handle
		if(s[4].equals("Restricted")) {
			body = new RestrictedBody2D(x, y, 0.5 * Math.PI);
		}
		else {
			body = new Body2D(x, y, 0.5 * Math.PI);
		}

		// The agent will have one body segment that is circular
		body.addSegment(new CircleSegment(0.0, 0.0, radius));

		// The agent will have one quarter circle
		sensors.add(new Sensor2D(0.0, 0.0, 10 * radius, 3 * Math.PI / 4, 5 * Math.PI / 4));
	}
	

	@Override
	public void rcvMessage(Object content, String senderId) {}


	@Override
	public void think() {

		final double x = ((Body2D)body).getX();
		final double y = ((Body2D)body).getY();
		double theta = 0.0;
		final double radius = ((Body2D)body).getRadius();


		// If the agent is outside of its boundary, make a 180 degree turn
		if(y + radius > MAX_Y || y - radius < MIN_Y) {
			theta = Math.PI;
		}

		// Move the given speed and rotation
		action = new AgentAction(speed, theta, 0.0);

		/*
		 *
		 * This code was used to send the ball's locations to a GUI that was
		 * running.  If simulating lots of agents, it's quicker to write their
		 * locations to a file while the app is running and then create a
		 * Java animation offline.
		 *
		try {
			Thread.sleep((long) 8);
		}
		catch (InterruptedException e) {}

		sendMessage(new AgentBallMsg(x, y, radius), senseThinkActDelay, "gui");
		 */

		// Go through the list of agents that were sensed this time step
		Map<String, Agent> sensedAgents = sense();
		for(String key : sensedAgents.keySet())
		{
			Agent a = sensedAgents.get(key);

			// TODO: Actually do something with the sensed agent
		}
	}


	/**
	 * Save the state of this FallingBall head. 
	 */
	@Override
	public Serializable saveHeadState() 
	{
		//The falling ball has no memory, so the head state is null
		return null;
	}


	/**
	 * Restore the state of this Agent's thoughts from a saved state.
	 * This agent has no memory, so there is no saved state here.
	 */
	@Override
	public void restoreHeadState(Serializable savedThoughts) 
	{
		assert (savedThoughts == null);
	}
}