package app;

import java.io.Serializable;
import sassy.monitoring_steering.NameValuePair;
import java.util.*;
import sassy.*;

 /** This class is used to test the monitoring and steering mechanism.
  *
  *@author Yin Xiong
  *@version 1.0 (02/2007)
  */


public class MS extends sassy.LogicalProcess implements Serializable
{
	/**flag for debugging*/
	final boolean MTRACE=true;

	/**class member for keeping track of how many messages have been sent.*/
	public int count;

	/**the application-specified name of the destination to which this LP will send messages to*/
	String dest;

	/**variable used for monitoring/steering purposes*/
	public int msInt;

	/**variable used for monitoring/steering purposes*/
	public double msDouble;

	/**variable used for monitoring/steering purposes*/
	public String msString;
	
	/**variable to keep the total of messages sent*/
	int total;

	/**Constructs a new Ping instance.*/
	public MS()
	{
		super();
		count = 0;
		msInt=0;
		msDouble=0.0;
		msString="*";
		total=100;
	}

	/**Sets application-specific configuration data.
	 * @param cdata String representing configuration data.
	 */
	@Override
	public void appSetConfigData(Serializable cdata)
	{
	
	}

	@Override
	public String toString()
	{
		return (getPID() + "-" + getLID() + " " + count);
	}

	/**Implements abstract method to save LP state*/
	@Override
	public Serializable appSaveLPState()
	{
		MS p = new MS();
		p.count = count;
		p.msInt=msInt;
		p.msDouble=msDouble;
		p.msString=msString;
		return p;
	}


	/**Implements abstract method to set the state of this LP to the state of a specified Ping*/
	public void appSetLPState(Serializable lp)
	{
		MS p = (MS)lp;
		count = p.count;
		msInt=p.msInt;
		msDouble=p.msDouble;
		msString=p.msString;
	} //end setState

	/**Initiates the LP and generates the initial message. It's the counterpart of "IProc"
	  *in GTW.
	  */
	@Override
	public void appInitializeLP()
	{
		System.out.println(getPID() + "-" + getLID() + " is initializing...\n");

		//create a message and send it to LP 1
		dest = getAPPid();
		int timeIncrement = 1;
		String cont=String.valueOf(count);
		sendSimMessage(cont, timeIncrement, dest);
		System.out.println(getAPPid()+ " sending initial msg...");
	}


	/**Handles events for Ping.
	  *It is the counterpart of "Proc" in GTW.
	  *@param content an Object representing the content of the message to be sent
	  *@param sender String the application-assigned name of the sender of this message
	  */
	@Override
	public void appRunLP(Object content, String sender)
	{

		try{
			Thread t=Thread.currentThread();
			Thread.sleep(1000);
		}
		catch(Exception e){
		}

		boolean endOfSim = false;

		//get message content which is the count of messages already sent
		String cont, newcontent;
		cont = (String)content;
		int timeIncrement = 1;
		newcontent = null;
		dest=getAPPid();

		count = Integer.parseInt(cont);
		if (count >= 0 && count < total)
		{
			count = count + timeIncrement;
			newcontent = String.valueOf(count);
			sendSimMessage(newcontent, timeIncrement, dest);
			msDouble++;
			msString=msString+"*";
		}

		else
		{
			setEndOfSimFlag();
		} //end if not end of sim message


		if(MTRACE)
			System.out.println("msString: " + msString);

		if (newcontent != null)
			System.out.println(getPID() + "-" + getLID() + ": receive " + cont + " send " + newcontent + "\n");

	}

	/**Terminates LP and displays simulation statistics. It is optionally called at the end of the
	  *simulation. It is the counterpart of "FProc" in GTW.
	  */
	@Override
	public void appFinalizeLP()
	{
		System.out.println(getPID() + "-" + getLID() + " is finalizing...\n");

	}

	/** Returns a Vector of NameValuePair describing all application-specific steerable
	 * parameters of this LP, each NameValuePair is in the form "paramName, paramType".
	 * @since 1.5
	 */
	@Override
	public Vector<NameValuePair> appGetSteerable()
	{
		Vector<NameValuePair> nvVector=new Vector<NameValuePair>();	
        	NameValuePair nv;
		nv=new NameValuePair("count", "int");
		nvVector.add(nv);
		nv=new NameValuePair("msDouble", "double");
		nvVector.add(nv);
		nv=new NameValuePair("msString", "String");
		nvVector.add(nv);
		return nvVector;

	}//end appGetSteerable


	/** Returns a Vector of NameValuePair describing all application-specific monitorable
	 * parameters of this LP, each NameValuePair is in the form "paramName, paramType".
	 * @since 1.5
	 */
	@Override
	public Vector<NameValuePair> appGetMonitorable()
	{
		Vector<NameValuePair> nvVector=new Vector<NameValuePair>();	
        	NameValuePair nv;
		nv=new NameValuePair("count", "int");
		nvVector.add(nv);
		nv=new NameValuePair("msDouble", "double");
		nvVector.add(nv);
		nv=new NameValuePair("msString", "String");
		nvVector.add(nv);
		return nvVector;
	}//end appGetMonitorable





} //end Ping
