package app;

import java.io.*;
import java.util.*;
import sassy.*;

/**This class provides a creator for the Phold simulation. The format of the configuration file is:
 * numLPs
 * @version 1.0 (04/2007)
 * @author Yin xiong
 */
public class PholdCreator implements Creator
{
	public List<PEInfo> appSetPEtable(int numPEs, List<LPConfigInfo> lpConfigInfoVector)
	{
		//using default creator
		DefaultCreator dc = new DefaultCreator();
		return dc.appSetPEtable(numPEs, lpConfigInfoVector);
	}


	/**Processes LP configuration information.
	 *@param fileName String the name of the configuration file
	 *@return a vector of LPConfigInfo object, each being the configuration information for an LP
	 */
	@Override
	public List<LPConfigInfo> appCreateLPs(String fileName)
	{
		List<LPConfigInfo> lpConfigInfoVector = new ArrayList<LPConfigInfo>();
		//Read configuration file to decide how many instance for each class need to be created
		try
		{
			String newLine = null;
			String dat = null;

			BufferedReader bf = new BufferedReader(new FileReader(fileName));

			int numLPs = 0;
			while ((newLine = bf.readLine()) != null)
			{
				//If the first character is a '#', this is a comment. Also, ignore blank lines
				if (newLine.trim().length() == 0 || newLine.charAt(0) == '#')
					continue;

				//Extract the type of line
				String args[] = newLine.split("\t");
				dat = newLine;
				numLPs = Integer.parseInt(args[0]);
			}//end while

			//create LPConfigInfo objects
			String appID, appName;
			String className = "app.Phold";
			for (int i = 0; i < numLPs; i++)
			{
				appID = String.valueOf(i);
				appName = "phold" + appID;
				LPConfigInfo lpcInfo = new LPConfigInfo(i, appID, appName, className, dat);
				lpConfigInfoVector.add(lpcInfo);
			} //end for

			bf.close();
		}
		catch (IOException e)
		{
			System.err.println("Error: Could not read configuration file:\n\t" + e.getMessage());
		}

		return lpConfigInfoVector;
	}

	@Override
	public String getTieBreakerClass()
	{
		return null;
	}
}//end class PholdCreator

