package app;

import sassy.monitoring_steering.NameValuePair;
import java.io.*;
import java.util.*;
import java.lang.*;
import java.sql.Time;
import sassy.*;

/**This class is the java-version of the Ping.c from GTW. It's used for testing the sim kernel.
 *
 *@author Yin Xiong
 *@version 2.0 (11/2006)
 */
public class PingPong extends sassy.LogicalProcess implements Serializable
{
	
	/**class member for keeping track of how many messages have been sent.*/
	int count;
	
	/**class member for store time increments*/
	int[] incs;
	
	/** class member for keeping track  of which time increment should be used */
	int incsCount;
	
	/**the application-specified name of the destination to which this LP will send messages to*/
	String dest;
	
	/**the time increment for messages sent by this LP; the message will be sent at currentTime+timeIncrement*/
	int timeIncrement;
	
	/**Constructs a new PingPong instance.*/
	public PingPong()
	{
		super();
		count=0;
		incsCount=0;
		timeIncrement=0;
	}
	
	/**Sets application-specific configuration data.
	 * @param cdata String representing configuration data.
	 */
	@Override
	public void appSetConfigData(Serializable cdata)
	{
		String[] s=((String)cdata).split(" ");
		int slength=s.length;
		dest=s[0];
		
		System.out.println("me, destination: " + getPID() + "-" + getLID() + " " + dest);
		incs=new int[slength-1];
		for(int i=1;i<slength-1;i++)
		{
			String ss=s[i];
			incs[i]=Integer.parseInt(ss);
		}
		
		for(int i=2;i<slength-2;i++)
			System.out.print(incs[i] + " ");
	}
	
	/**Implements abstract method to save LP state*/
	@Override
	public Serializable appSaveLPState()
	{
		PingPong p=new PingPong();
		p.count=count;
		
		return p;
	}
	
	
	/**Implements abstract method to set the state of this LP to the state of a specified Ping*/
	@Override
	public void appSetLPState(Serializable lp)
	{
		PingPong p=(PingPong)lp;
		count=p.count;
		
		
	}//end setState
	
	/**Initiates the LP and generates the initial message. */
	@Override
	public void appInitializeLP()
	{
		System.out.println(getPID() + "-" + getLID() + " initializing...\n");
		
		//create a message and send it to LP 1
		count=1;
		String cont=String.valueOf(count);
		timeIncrement=incs[incsCount];
		incsCount++;
		sendSimMessage(cont, timeIncrement, dest);
		System.out.println(getPID() + "-" + getLID() + " is sending initial msg...");
	}
	
	
	/**Handles events for PingPong.
	 *It is the counterpart of "Proc" in GTW.
	 *@param content an Object representing the content of the message to be sent
	 *@param sender String the application-assigned name of the sender of this message
	 */
	public void appRunLP(Object content, String sender)
	{
		boolean endOfSim=false;
		
		//get message content which is the count of messages already sent
		String cont, newcontent;
		cont=(String)content;
		timeIncrement= 1;//incs[incsCount];
		incsCount++;
		
		newcontent=null;
		
		
		count=Integer.parseInt(cont);
		if(count>=0 && count<15)
		{ //send 10 messages in total
			//count++;
			count=count+timeIncrement;
			newcontent=String.valueOf(count);
			//new: sendSimMessage(Object content, double timestamp, String targetid)
			sendSimMessage(newcontent, timeIncrement, dest);
		}//end if count<10
		
		
		else
		{
			setEndOfSimFlag();
		}//end if not end of sim message
		
		
		if(newcontent!=null)
			System.out.println("\n" +getPID() + "-" + getLID() + " receive " + cont + " send " + newcontent);
		
	}
	
	/**Terminates LP and displays simulation statistics. It is optionally called at the end of the
	 *simulation. It is the counterpart of "FProc" in GTW.
	 */
	public void appFinalizeLP()
	{
		System.out.println(getPID() + "-" + getLID() + " is finalizing...\n");
		
	}
	/** Returns a Vector of NameValuePair describing all application-specific steerable
	 * parameters of this LP, each NameValuePair is in the form "paramName, paramType".
	 * @since 1.5
	 */
	public Vector<NameValuePair> appGetSteerable()
	{
		Vector<NameValuePair> nvVector=new Vector<NameValuePair>();
		NameValuePair nv;
		nv=new NameValuePair("count", "int");
		nvVector.add(nv);
		return nvVector;
	}//end appGetSteerable
	
	
	/** Returns a Vector of NameValuePair describing all application-specific monitorable
	 * parameters of this LP, each NameValuePair is in the form "paramName, paramType".
	 * @since 1.5
	 */
	public Vector<NameValuePair> appGetMonitorable()
	{
		Vector<NameValuePair> nvVector=new Vector<NameValuePair>();
		NameValuePair nv;
		nv=new NameValuePair("count", "int");
		nvVector.add(nv);
		return nvVector;
	}//end appGetMonitorable
	
}//end PingPong
