package app;

import java.io.Serializable;
import sassy.monitoring_steering.NameValuePair;
import java.util.*;
import sassy.*;
/**This class is the java-version of the Ping.c from GTW. It's used for testing the sim kernel.
  *
  *@author Yin Xiong
  *@version 1.7 (07/2006)
  */


public class Pong extends sassy.LogicalProcess implements Serializable
{
	/**class member for keeping track of how many messages have been sent.*/
	int count;

	/**the application-specified name of the destination to which this LP will send messages to*/
	String dest;

	/**the time increment for messages sent by this LP; the message will be sent at currentTime+timeIncrement*/
	int timeIncrement;

	/**Constructs a new Pong instance.*/
	public Pong()
	{
		super();
		count = 0;
		timeIncrement = 0;
	}


	/**Sets application-specific configuration data.
	 * @param cdata String representing configuration data.
	 */
	@Override
	public void appSetConfigData(Serializable cdata)
	{
//		String[] s = cdata.split(" ");
//		int slength = s.length;
//		//System.out.println(alias + " string length: " + slength);
//		int[] incs = new int[slength];
//		for (int i = 0;i < slength;i++)
//		{
//			String ss = s[i];
//			incs[i] = Integer.parseInt(ss);
//		}

		//for(int i=0;i<slength;i++)
		//System.out.println(incs[i]);
	}


	@Override
	public String toString()
	{
		return (getPID() + "-" + getLID() + " " + count);
	}

	/**Implements abstract method to save LP state*/
	@Override
	public Serializable appSaveLPState()
	{
		Pong p = new Pong();
		p.count = count;

		return p;
	}

	/**Implements abstract method to set the state of this LP to the state of a specified Pong*/
	@Override
	public void appSetLPState(Serializable lp)
	{
		Pong p = (Pong)lp;
		count = p.count;

	} //end setState

	/**Initiates the LP. Pong does not need to send out initial message. Ping has alread done that.
	  *It's the counterpart of "IProc" in GTW.
	  */
	@Override
	public void appInitializeLP()
	{
		System.out.println(getPID() + "-" + getLID() + " is initializing...\n");

		//Pong does not need to send out initial message.
	}


	/**Handles events for Pong.
	  *It is the counterpart of "Proc" in GTW.
	  *@param content an Object representing the content of the message to be sent                                          
	  *@param sender String the application-assigned name of the sender of this message                                     
	  */
	@Override
	public void appRunLP(Object content, String sender)
	{
		boolean endOfSim = false;

		//get message content which is the count of messages already sent
		String cont, newcontent;
		cont = (String)content;
		newcontent = null;
		timeIncrement = 1; //time increment
		dest = "ping";


		count = Integer.parseInt(cont);
		if (count >= 0 && count <= 10)
		{ //send 10 messages in total
			//count++;
			count = count + timeIncrement;
			newcontent = String.valueOf(count);
			sendSimMessage(newcontent, timeIncrement, dest);

		} //end if

		if (newcontent != null)
			System.out.println(getPID() + "-" + getLID() + " receive " + cont + " send " + newcontent + "\n");
	}

	/**Terminates LP and displays simulation statistics. It is optionally called at the end of the
	  *simulation. It is the counterpart of "FProc" in GTW.
	  */
	@Override
	public void appFinalizeLP()
	{
		System.out.println(getPID() + "-" + getLID() + " is finalizing...\n");

	}

	 /** Returns a Vector of NameValuePair describing all application-specific steerable
         * parameters of this LP, each NameValuePair is in the form "paramName, paramType".
         * @since 1.5
         */     
	@Override
        public Vector<NameValuePair> appGetSteerable()
        {
                Vector<NameValuePair> nvVector=new Vector<NameValuePair>();
                NameValuePair nv;
                nv=new NameValuePair("count", "int");
                nvVector.add(nv);
                return nvVector;
        }//end appGetSteerable
        
         
        /** Returns a Vector of NameValuePair describing all application-specific monitorable
         * parameters of this LP, each NameValuePair is in the form "paramName, paramType".
         * @since 1.5
         */      
	@Override
        public Vector<NameValuePair> appGetMonitorable()
        {
                Vector<NameValuePair> nvVector=new Vector<NameValuePair>();
                NameValuePair nv;
                nv=new NameValuePair("count", "int"); 
                nvVector.add(nv);
                return nvVector;
        }//end appGetMonitorable
} //end Ping
