package app;

import java.util.Comparator;
import java.io.Serializable;
import sassy.Message;

/**
 * <p>To ensure repeatability, when there are messages with equal time stamps in a 
 * simulation, they must be processed in the same order. When the SASSY kernel
 * encounters two distinct messages which have the time stamp, it calls this
 * application-specified comparator to decide which message should go first.</p>
 *
 * <p>For repeatability it is necessary that the application-specified comparator decides
 * which message goes first independently of the current state of the simulation.</p>
 *
 * <p>This comparator returns -1, +1, or 0. 
 * -1 means that the first message should go first,
 * 1 means that the second message should go first. If the comparator returns 0,
 * the SASSY kernel will use other tie-breaking parameters to ensure that simultaneous
 * events are processed in a repeatable manner.</p>
 *
 * @author George Vulov <georgevulov@hotmail.com>
 *
 * TODO: Implement Serializable also
 */
public class TieBreakingComparator implements Serializable, Comparator <Message>
{
	private static final long serialVersionUID = 3823437040342L;

	/**
	 * Break simultaneous events in Ping-Pong & record their occureence.
	 *
	 * @param m1 The first message
	 * @param m2 The second message, with equal time stamp to the first
	 * @return negative if m1 should go first, positive if m2 should go first, 0 if can't decide
	 */
	public int compare(Message m1, Message m2)
	{
		System.out.println("Ping-Pong tie-breaking comparator called!");
		
		//Ping should always go first
		if (m1.getSenderAPPid().equalsIgnoreCase("ping") && m2.getSenderAPPid().equalsIgnoreCase("pong"))
			return -1;
		else if (m1.getSenderAPPid().equalsIgnoreCase("pong") && m2.getSenderAPPid().equalsIgnoreCase("ping"))
			return 1;
		
		//That was our only rule. Since we can't decide, let the kernel decide
		return 0;
	}
}
