package app.antsim;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import sassy.Creator;
import sassy.LPConfigInfo;
import sassy.PEInfo;

/**
 * Instantiates all the ants in the environment.
 * 
 * @author George Vulov <georgevulov@hotmail.com>
 */
public class AntEnvironmentCreator implements Creator
{	
	public static class InitInfo implements Serializable
	{
		double x;
		double y;
		double theta;
	}
	
	@Override
	public String getTieBreakerClass()
	{
		return null;
	}


	@Override
	public List<LPConfigInfo> appCreateLPs(String fileName)
	{
		List<LPConfigInfo> allLPs = new ArrayList<LPConfigInfo>(SimParams.ANT_COUNT);
		int agentCount = 0;
		
        //Instantiate all the ants
		List<String> antIds = new ArrayList<String>(SimParams.ANT_COUNT);
		for (int i = 0; i < SimParams.ANT_COUNT; i++)
		{
            InitInfo antParams = new InitInfo();        
			antParams.x = SimParams.NEST_X + (0.5 - Math.random()) * SimParams.PHEROMONE_AGENT_WIDTH;
			antParams.y = SimParams.NEST_Y + (0.5 - Math.random()) * SimParams.PHEROMONE_AGENT_WIDTH;
			antParams.theta = Math.random() * 2 * Math.PI;
			
			String name = "Ant " + i;
			antIds.add(name);
			LPConfigInfo antLP = new LPConfigInfo(agentCount++, name, name, "app.antsim.AntAgent", antParams);
			allLPs.add(antLP);
		}
		
		//Instantiate all the pheromone agents
		assert SimParams.MAX_X % SimParams.PHEROMONE_AGENT_WIDTH == 0;
		assert SimParams.MAX_Y % SimParams.PHEROMONE_AGENT_WIDTH == 0;
		int rowCount = SimParams.MAX_Y / SimParams.PHEROMONE_AGENT_WIDTH;
		int colCount = SimParams.MAX_X / SimParams.PHEROMONE_AGENT_WIDTH;
		for (int col = 0; col < colCount; col++)
		{
			for (int row = 0; row < rowCount; row++)
			{
				PheromoneAgent.BodyInfo pheromoneConfig = new PheromoneAgent.BodyInfo();
				pheromoneConfig.xMin = col * SimParams.PHEROMONE_AGENT_WIDTH;
				pheromoneConfig.yMin = row * SimParams.PHEROMONE_AGENT_WIDTH;
				pheromoneConfig.width = SimParams.PHEROMONE_AGENT_WIDTH;
				
				//Calculate the neighbors
				for (int c = col - 1; c <= col + 1; c++)
				{
					for (int r = row - 1; r <= row + 1; r++)
					{
						if (c >= 0 && c < colCount && r >= 0 && r < rowCount)
						{
							//Don't add the agent itself as its own neighbor
							if (r != row || c != col)
								pheromoneConfig.neighborIds.add(getNameForPheromoneBlock(r, c));
						}
					}
				}
				
				//Each pheromone agent also updates all the ants
				pheromoneConfig.initialSubscribers = antIds;
				
				String name = getNameForPheromoneBlock(row, col);
				LPConfigInfo lp = new LPConfigInfo(agentCount++, name, name, "app.antsim.PheromoneAgent", pheromoneConfig);
				allLPs.add(lp);
			}
		}
        
        //Create the GUI to display the ant movement
        LPConfigInfo guiLP = new LPConfigInfo(agentCount++, "gui", "Ant GUI", "app.antsim.AntGUI", null);
        allLPs.add(guiLP);
		
		return allLPs;
	}

	private String getNameForPheromoneBlock(int row, int col)
	{
		return String.format("Pheromone Block (row=%d, col=%d)", row, col);
	}

	@Override
	public List<PEInfo> appSetPEtable(int numPEs, List<LPConfigInfo> lpConfigInfoVector)
	{
		//Allocate all the PE's
		assert numPEs > 0;
		List<PEInfo> executives = new ArrayList<PEInfo>(numPEs);
		for (int i = 0; i < numPEs; i++)
		{
			executives.add(new PEInfo());
		}
		
		//Let's just do a round-robin allocation of the ants to the processing executives
		int currentPE = 0;
		for (LPConfigInfo lpInfo : lpConfigInfoVector)
		{
			executives.get(currentPE).lpcVector.add(lpInfo);
			currentPE = (currentPE + 1) % executives.size();
		}
		
		return executives;
	}
}
