package app.antsim;

import sassy.agent.*;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.*;
import javax.swing.JFrame;
import javax.swing.JPanel;
import sassy.LogicalProcess;
import java.awt.BorderLayout;
import java.awt.Graphics2D;
import java.io.Serializable;
import java.util.concurrent.ConcurrentHashMap;
import javax.swing.BorderFactory;
import javax.swing.JSlider;
import javax.swing.Timer;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class AntGUI extends LogicalProcess implements Serializable
{
	/** The message received by an ant. */
	public static class AntInfo implements Serializable
	{
		public AntInfo(double x, double y, double theta, double r, boolean hasFood)
		{
			this.x = x;
			this.y = y;
			this.theta = theta;
			radius = r;
			isCarryingFood = hasFood;
		}
		
		public double x;

		public double y;
		
		/** Direction in which the ant is facing. */
		public double theta;

		public double radius;
		
		public boolean isCarryingFood;
	}
	
	public boolean endOfSimulation = false;

	/** Save the last message from each ant. */
	public ConcurrentHashMap<String, AntInfo> ants = new ConcurrentHashMap<String, AntInfo>();
	
	public ConcurrentHashMap<String, PheromoneAgent.UpdateMessage> pheromones = 
					new ConcurrentHashMap<String, PheromoneAgent.UpdateMessage>();


	@Override
	public void appFinalizeLP()
	{
		endOfSimulation = true;
	}


	@Override
	public void appInitializeLP()
	{
		AntGUIFrame guiFrame = new AntGUIFrame();
	}


	@Override
	public void appRunLP(Object agentMessage, String sender)
	{
		Serializable content = ((AgentMessage)agentMessage).getContent();
		
		if (content instanceof AntInfo)
		{
			AntInfo ball = (AntInfo) content;
			ants.put(sender, ball);
		}
		else if (content instanceof PheromoneAgent.UpdateMessage)
		{
			pheromones.put(sender, (PheromoneAgent.UpdateMessage)content);
		}
	}

	/** We use this class to save our state (for rollbacks.) */
	private final static class SavedState implements Serializable
	{
		ConcurrentHashMap<String, AntInfo> ants;
		ConcurrentHashMap<String, PheromoneAgent.UpdateMessage> pheromones;
	}

	@Override
	public Serializable appSaveLPState()
	{
		SavedState s = new SavedState();
		s.ants = new ConcurrentHashMap<String, AntInfo>(ants);
		s.pheromones = new ConcurrentHashMap<String, PheromoneAgent.UpdateMessage>(pheromones);
		return s;
	}


	@Override
	public void appSetConfigData(Serializable cdata)
	{
		assert cdata == null;
	}


	@Override
	@SuppressWarnings("unchecked")
	public void appSetLPState(Serializable lp)
	{
		SavedState s = (SavedState)lp;
		ants = s.ants;
		pheromones = s.pheromones;
	}

	class AntGUIFrame extends JFrame
	{
		public AntGUIFrame() throws HeadlessException
		{
			super("Ant Simulator");
			setSize(SimParams.MAX_X, SimParams.MAX_Y + 100);
			setResizable(false);
			setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			getContentPane().add(new AntApplicationPanel());
			pack();
			setVisible(true);
		}
	}


	class AntApplicationPanel extends JPanel
	{
		AntPanel ballPanel;

		private javax.swing.JPanel buttonPanel;

		private javax.swing.JScrollPane jScrollPane1;

		private javax.swing.JSlider delaySlider;

		private javax.swing.JTextArea jTextArea1;

		private javax.swing.JButton startButton;

		private javax.swing.JButton stopButton;


		/** Creates new form NewFrame */
		public AntApplicationPanel()
		{
			initComponents();
		}


		private void initComponents()
		{
			ballPanel = new AntPanel();
			ballPanel.run();
			jScrollPane1 = new javax.swing.JScrollPane();
			jTextArea1 = new javax.swing.JTextArea();
			buttonPanel = new javax.swing.JPanel();
			startButton = new javax.swing.JButton();
			stopButton = new javax.swing.JButton();


			delaySlider = new javax.swing.JSlider(JSlider.HORIZONTAL, 1, 200, 200);
			delaySlider.setBorder(BorderFactory.createTitledBorder("Print Update Intervel"));
			delaySlider.setInverted(true);
			delaySlider.setMajorTickSpacing(50);
			delaySlider.setPaintTicks(true);
			delaySlider.addChangeListener(new ChangeListener()
			{
				@Override
				public void stateChanged(ChangeEvent evt)
				{
					ballPanel.interval = delaySlider.getValue();
				}
			});

			jTextArea1.setColumns(20);
			jTextArea1.setRows(5);
			jScrollPane1.setViewportView(jTextArea1);

			startButton.setText("Nest Pheromones");
			startButton.addActionListener(new java.awt.event.ActionListener()
			{
				@Override
				public void actionPerformed(java.awt.event.ActionEvent evt)
				{
					nestPheromonesButtonActionPerformed(evt);
				}
			});
			buttonPanel.add(startButton);

			stopButton.setText("Food Pheromones");
			stopButton.addActionListener(new java.awt.event.ActionListener()
			{
				@Override
				public void actionPerformed(java.awt.event.ActionEvent evt)
				{
					foodPheromonesButtonActionPerformed(evt);
				}
			});
			buttonPanel.add(stopButton);

			delaySlider.addChangeListener(new javax.swing.event.ChangeListener()
			{
				@Override
				public void stateChanged(javax.swing.event.ChangeEvent evt)
				{
					jSlider1StateChanged(evt);
				}
			});
			buttonPanel.add(delaySlider);
			this.setLayout(new BorderLayout());
			add(buttonPanel, java.awt.BorderLayout.NORTH);
			add(ballPanel, java.awt.BorderLayout.SOUTH);


		}


		private void jSlider1StateChanged(javax.swing.event.ChangeEvent evt)
		{
			ballPanel.mTimer.setDelay(delaySlider.getValue());
		}

		private void foodPheromonesButtonActionPerformed(java.awt.event.ActionEvent evt)
		{
			ballPanel.showFood = true;
		}

		private void nestPheromonesButtonActionPerformed(java.awt.event.ActionEvent evt)
		{
			ballPanel.showFood = false;
		}
	}

	class AntPanel extends JPanel
	{
		/** Update interval, in milliseconds. */
		private int interval = 100;
		
		/** If true, we show food pheromones. Else, we show home pheromones. */
		public boolean showFood = false;

		public Timer mTimer;

		public AntPanel()
		{
			setPreferredSize(new Dimension(SimParams.MAX_X, SimParams.MAX_Y));
			setBorder(BorderFactory.createLineBorder(Color.BLACK));
		}


		public void run()
		{
			repaint();
			mTimer = new Timer(interval, new TimerAction());
			mTimer.start();
		}


		public void setAnimation(boolean turnOnOff)
		{
			if (turnOnOff)
				mTimer.start();  // start animation by starting the timer.
			else
				mTimer.stop();   // stop timer
		}

		class TimerAction implements ActionListener
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				if (!AntGUI.this.endOfSimulation)
				{
					repaint();
				}
				else
				{
					mTimer.stop();
				}
			}
		}


		@Override
		public void paintComponent(Graphics gr)
		{
			Graphics2D graphics = (Graphics2D)gr;
			super.paintComponent(graphics);  // Paint background, border

			//Draw the pheronomes.
			//First, find the maximum pheronome value
			double foodMax = 0, homeMax = 0;
			for (PheromoneAgent.UpdateMessage pheromone : AntGUI.this.pheromones.values())
			{
				foodMax = Math.max(foodMax, pheromone.foodPheromone);
				homeMax = Math.max(homeMax, pheromone.homePheromone);
			}
			
			//Now, draw each pheromone cell
			for (PheromoneAgent.UpdateMessage pheromone : AntGUI.this.pheromones.values())
			{
				double val = pheromone.homePheromone / homeMax;
				if (showFood)
				{
					val = pheromone.foodPheromone / foodMax;
				}
				val = Math.min(val, 1);
								
				//Create the color. If we're showing food we're red, if showing nest we're green
				float r = 0, g = 0, b = 0;
				if (showFood)
					r = (float)val;
				else
					g = (float)val;
				
				Color color = new Color(r, g, b);
				graphics.setColor(color);
				
				graphics.fillRect(pheromone.x, pheromone.y, pheromone.width, pheromone.width);
			}
			
			//Draw ants on top of the pheromones
			for (AntInfo ant : AntGUI.this.ants.values())
			{
				graphics.setColor(Color.BLUE);
				int r = (int)ant.radius;

				graphics.rotate(ant.theta, ant.x, ant.y);
				//Draw body
				graphics.fillOval((int) (ant.x - r), (int) (ant.y - r / 2), (2 * r), r);
				//Draw head
				if (ant.isCarryingFood)
					graphics.setColor(Color.GREEN);
				else
					graphics.setColor(Color.RED);

				graphics.fillOval((int)(ant.x + r/2) + 2, (int) (ant.y - r/ 2) + 2, r - 4, r - 4);
				graphics.rotate(-ant.theta, ant.x, ant.y);
			}
		}
	}
}