package im;

import java.awt.Point;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
public class SimpleBallSimulator
{
	int totalMove = 0;
	public static HashMap<String, SimpleBall> sharedBallList = new HashMap<String, SimpleBall>();

	/**
	 * @param args
	 */
	public static void main(String[] args)
	{
		SimpleBallSimulator simulator = new SimpleBallSimulator();
		simulator.readConfigFile("im/config_ball");
		long startTime = System.currentTimeMillis();
		simulator.runSim();
		long endTime = System.currentTimeMillis();
		long exeTime = endTime - startTime;
		
		System.out.println("Execution time: "+exeTime);
	}
	
	private void readConfigFile(String fileName)
	{
//		Read configuration file to decide how many instance for each class need to be created
		try
		{
			String newLine = null;
			BufferedReader bf = new BufferedReader(new FileReader(fileName));

			//Each line is of the format: "<line type>:type-specific info"
			//LP's are specified as "LP:<id> <name> <class name> <application data>"
			while ((newLine = bf.readLine()) != null)
			{
				//If the first character is a '#', this is a comment. Also, ignore blank lines
				if (newLine.trim().length() == 0 || newLine.charAt(0) == '#')
					continue;

				//Extract the type of line
				int pos = newLine.indexOf(':');
				if (pos == -1)
				{
					bf.close();
					System.err.println("Error1: Malformated configration file.");
					System.exit(1);
				}
				String lineType = newLine.substring(0, pos).trim();
				newLine = newLine.substring(pos + 1).trim();

				//Now, process the line depending on what type it is
				if (lineType.equalsIgnoreCase("LP"))
				{
					
					//Only split on the first 3 fields because the application data can have
					String[] tokens = newLine.split("\\s+", 4);

					if (tokens.length < 3)
					{
						bf.close();
						System.err.println("Error: Malformated configration file.");
						System.exit(1);
					}

					String id = tokens[0].trim();
					String name = tokens[1].trim();
					String className = tokens[2].trim();
					String dat = tokens.length > 3 ? tokens[3] : "";
					
					if(newLine.contains("im.Ball")){
						SimpleBall ball = new SimpleBall();
						
						String[] s = newLine.split(" ");
						
						ball.APPid = s[1];
						
						final int startIndex = 9;
						ball.radius = Integer.parseInt(s[startIndex]);
						ball.envi = new Point(Integer.parseInt(s[startIndex+1]), Integer.parseInt(s[startIndex+2]));
						ball.numberGrid = Integer.parseInt(s[startIndex+3]);
						ball.currPosition.x = Integer.parseInt(s[startIndex+4]);
						ball.currPosition.y = Integer.parseInt(s[startIndex+5]);
						ball.nextPosition.x = ball.currPosition.x;
						ball.nextPosition.y = ball.currPosition.y;
						ball.currVelocity.x = Integer.parseInt(s[startIndex+6]);
						ball.currVelocity.y = Integer.parseInt(s[startIndex+7]);
						ball.nextVelocity.x = ball.currVelocity.x;
						ball.nextVelocity.y = ball.currVelocity.y;
						
						ball.updateIntervel = Integer.parseInt(s[startIndex+8]);
						ball.totalMove = Integer.parseInt(s[startIndex+9]);
						totalMove = ball.totalMove;
						ball.numBalls = Integer.parseInt(s[startIndex+10]);
						ball.terAddr = s[startIndex+11];
//						ball.type = s[startIndex+12];
						
						sharedBallList.put(ball.getAPPid(),ball);
//						
					}					
				}
			} //end while

			bf.close();
		}
		catch (IOException e)
		{
			System.err.println("Error: Could not read configuration file:\n\t" + e.getMessage());
		}
//		System.out.println("*****Size of the LPs:"+lpConfigInfoVector.size());
//		return sharedBallList;
	}

	private void runSim()
	{
		for(int j=0;j<totalMove;j++){
		
			for(SimpleBall ball: sharedBallList.values())
			{
				Point newPosition = ball.move();
				sharedBallList.get(ball.APPid).setCurrPosition(newPosition);
				for(SimpleBall ballUpdte:sharedBallList.values())
				{
					if(!ballUpdte.APPid.equals(ball.getAPPid()))
					{
						ballUpdte.update(new SimpleBallMsg(ball.getAPPid(),newPosition));
					}
				}
			}
		}
	}

}
