package sassy;

import java.util.*;

/**This class is responsible to get and process simulation configuration information from command line or
 * from a file and store the information in LPConfigInfo objects. Simulation application programmers can
 * provide their own creator by creating a class that extends the Creator class and implementing the 
 * abstract method appCreateLPs or they can use the default creator.
 *
 * @author George Vulov (georgevulov@hotmail.com)
 * @author Yin xiong
 */

public interface Creator
{
	/**
	 * Return the simultaneous event comparator to be used in the simulation. Return null
	 * in order to use an arbitrary (but consistent) rule for breaking ties.
	 * 
	 * @return full name (including packages) of the tie-breaking comparator class
	 */
	public String getTieBreakerClass();

	/**Processes LP configuration information.
	  *@param fileName String the name of the configuration file
	  *@return a vector of LPConfigInfo object, each being the configuration information for an LP
	  */
	public List<LPConfigInfo> appCreateLPs(String fileName);

	/**Allocates LPs to PEs
	  *@param numPEs int the number of PEs involved in this simulation.
	  *@param lpConfigInfoVector the LPs involved in this simulation.
	  *@return a Vector of PEInfo objects.
	  */
	public List<PEInfo> appSetPEtable(int numPEs, List<LPConfigInfo> lpConfigInfoVector);
} 
