package sassy;

import java.rmi.*;

/**
 * This is the interface for Global Name Server(GNS).
 * 
 * @author Yin Xiong
 * @author George Vulov <georgevulov@hotmail.com>
 */
public interface GNSInterface extends java.rmi.Remote
{
	/**
	 * RMI method which registers a new PE and returns the LP configuration
	 * information to the new PE.
	 *
	 * @param ip the ip of the PE
	 * @param machineName the machine name of the PE
	 * @param peerStub the RMI remote reference for the PE
	 * @return the LP's that are assigned to the PE
	 */
	public GNS.PEConfig registerPE(String ip, String machineName, Peer peerStub) throws RemoteException;

	/**Gets the routing information for a message by the application-specified id.
	  *@param aid String the application-specified id of the receiver of the message.
	  */
	public RoutingInfo getRoutingInfoByAid(String aid) throws RemoteException;

	/**Gets the routing information for a message by the system-assigned LP id.
	  *@param lid int the LP id of the receiver of this message.
	  */
	public RoutingInfo getRoutingInfoByLid(int lid) throws RemoteException;

	/**
	 * Returns whether the simulation has ended or not.
	 *
	 * @return true if the simulation is still running, false if it has been terminated
	 */
	public boolean isSimulationRunning() throws RemoteException;

	/**
	 * Sets the end of simulation flag. All Process Executives will be
	 * terminated and the simulation will end.
	 */
	public void endSimulation() throws RemoteException;

	/**
	 * Requests that the GNS initiate a calculation of Global Virtual Time (GVT).
	 * GVT guarantees that no messages with timestamp < GVT will occur, thus
	 * allowing logical processes to free rollback history for such messages.
	 */
	public void requestGVTCalculation() throws RemoteException;

	/**
	 * Receives a monitoring/steering message from a remote clients.
	 * @param msg  a Message instance to be received.
	 */
	public void receive_MS_Msg(Message msg) throws RemoteException;

	/**Gets the flag for collecting statistics*/
	public boolean isCollectingStats() throws RemoteException;
}


