package sassy;

import java.io.*;

/**This class stores the configuration information for an LP. The information is provided
 *by application programmer in a text file in the following format: <br>
 *index app_name class_name data
 *
 *@version 2.0 (05/2007)
 *@author Yin Xiong
 */

public class LPConfigInfo implements Serializable
{
	/**the id of the PE which the message is routing to*/
	int PEid;

	/**the id of the LP which the message is routing to*/
	int LPid;

	/**the application-specified index of the LP*/
	private String APPid;

	/**the application-specified name of the LP*/
	private String APPname;

	/**the class name of the LP*/
	private String className;

	/** LP-specific data object that will be passed to the LP handler when the LP is initialized. */
	private Serializable data;

	/**constructs a LPInfo object*/
	public LPConfigInfo(int uniqueLPid, String uniqueApplicationId, String displaynName, String className, Serializable initData)
	{
		LPid = uniqueLPid;
		APPid = uniqueApplicationId;
		APPname = displaynName;
		this.className = className;
		data = initData;
	} //end constructor

	/**Gets index of the LP*/
	public String getAPPid()
	{
		return APPid;
	}

	/**Gets the name of the LP*/
	public String getAPPname()
	{
		return APPname;
	}

	/**Gets the class name of the LP*/
	public String getLPClassName()
	{
		return className;
	}

	/**Gets the data of the LP*/
	public Serializable getLPData()
	{
		return data;
	}

	@Override
	public String toString()
	{
		String s = " " + PEid + " " + LPid + " " + APPid + " " + APPname + " " + className + " " + data;
		return s;
	}
} //end LPConfigInfo
