package sassy;

import java.io.*;

/**This class holds the statistics for a single LP.
 *
 * @version 1.0 (03/2007)
 * @author Yin xiong
 */

public class LPStat implements Serializable, Comparable<LPStat>
{

	/** the id of the PE this LPStat resides on*/
	int PEid;

	/** the id of the LP whose statistics is stored in this LPStat instance*/
	int LPid;
	
	/** Unique application-assigned identifier of this LP.*/
	String APPid;

	/** the time the LP begins to run*/
	long beginTime = 0;

	/** the time duration of this LP*/
	long simTime = 0;

	/** the time the LP stops to run*/
	long endTime = 0;

	/** The number events rolled back. */
	int numEventsRolledBack = 0;
	
	/** Number of rollbacks occurred (different length rollbacks all count as 1). */
	int numRollbacks = 0;
	
	/** Number of anti-messages not sent out because of lazy cancellation. */
	int numAntiMessagesLazilyCancelled = 0;

	/** the number of events this LP has processed*/
	int numProcEvents = 0;

	/** the number of events this LP has committed*/
	int numCommEvents = 0;

	/** the number of events canceled*/
	int numCanceledEvents = 0;

	/** the number of anti messages has been sent by this LP*/
	int numAntiMsgsOut;

	/** the number of anti messages has been received by this LP*/
	int numAntiMsgsIn;

	/** the number of monitoring messages*/
	int numMonitoringMsgs = 0;

	/** the number of steering messages*/
	int numSteeringMsgs = 0;

	/** the number of fossil collections*/
	int numFossils = 0;

	/** the longest rollback*/
	int longestRollback = 0;

	/** average depth of rollback*/
	double avgRBLength = 0.0;

	/**Returns a string value of this object.*/
	@Override
	public String toString()
	{
		if (numEventsRolledBack != 0)
			avgRBLength = (double)numAntiMsgsOut / numEventsRolledBack;

		simTime = endTime - beginTime;

		String s = "\nID:       " + PEid + "\t" + LPid
			   + "\nTime:     " + beginTime + "\t" + endTime + "\t" + simTime
			   + "\nEvents:   " + numProcEvents + "\t" + numEventsRolledBack + "\t" + numCommEvents + "\t"
			   + "\t" + numCanceledEvents
			   + "\nMessages: " + + numAntiMsgsIn + "\t" + numAntiMsgsOut + "\t"
			   + numMonitoringMsgs + "\t" + numSteeringMsgs
			   + "\nOthers:   " + numFossils + "\t" + avgRBLength + "\t" + longestRollback;
		return s;
	} //end toString

	/**Gets the APPid */
	String getAPPid()
	{
		return APPid;
	}
	
	/**Sets the APPid */
	void setAPPid(String aid)
	{
		APPid = aid;
	}
	
	/**Gets the LPid*/
	int getLPid()
	{
		return LPid;
	}

	/**Sets the PEid*/
	void setPEid(int pid)
	{
		PEid = pid;
	} //end setPEid

	/**Sets the LPid*/
	void setLPid(int lid)
	{
		LPid = lid;
	} //end setLPid

	/**Compares two LPStat instances*/
	public int compareTo(LPStat o)
	{
		if (PEid < o.PEid)
			return -1;
		else if (PEid > o.PEid)
			return 1;
		else
		{
			if (LPid < o.LPid)
				return -1;
			else
				return 1;
		} //end else
	} //end compareTo


	public boolean equals(Object o)
	{
		if(o == null || !(o instanceof LPStat))
			return false;

		LPStat lp = (LPStat) o;
		return PEid == lp.PEid && LPid == lp.LPid;
	}


	public int hashCode() {
		return 1000000 * PEid + LPid;
	}

} //end class
