package sassy;

import java.util.*;

/**
 * This class represents the local name server that routes the messages among PEs. Each
 * PE has a related local name server.
 * @author George Vulov
 * @author Yin Xiong
 */
public class LocalNameServer
{
	/** Map of LP ID to the Routing information */
	private final Map<Integer, RoutingInfo> lpIdToRoutingInfo = new HashMap<Integer, RoutingInfo>();

	/** Map of Application ID to Routing Information */
	private final Map<String, RoutingInfo> appIdToRoutingInfo = new HashMap<String, RoutingInfo>();

	/**Looks up the routing info by an LP's application id.
	 *@param appid String representing the application id of the LP.
	 */
	public RoutingInfo getRoutingInfoByAid(String appid)
	{
		return appIdToRoutingInfo.get(appid);
	}


	/**Looks up the routing info by an LP id.
	 *@param lid int representing the LP id.
	 */
	public RoutingInfo getRoutingInfoByLid(int lid)
	{
		return lpIdToRoutingInfo.get(lid);
	}


	/**Adds a row of routing information into the routing table.
	 *@param rinfo an RoutingInfo instance representing a row in the table.
	 *@return the size of the routing table.
	 */
	void addRoutingInfo(RoutingInfo rinfo)
	{
		lpIdToRoutingInfo.put(rinfo.getLPid(), rinfo);
		appIdToRoutingInfo.put(rinfo.getAPPid(), rinfo);
	}
}
