package sassy;

import java.util.*;

/**
 * This class represents a table row in the PE information table maintained
 * by the Global Name Server.
 * @version 2.0
 * @since 2.0
 * @author Yin Xiong(10/2006)
 * @author George Vulov (georgevulov@hotmail.com)
 */

public class PEInfo
{
	/**
	 * The RMI remote reference used to communicate with this PE.
	 */
	private Peer rmiPeerStub;

	/**The GNS-assigned integer ID to this PE.*/
	private int PEid;

	/**The ip represented as a String*/
	private String IP;

	/**The name of the machines on which this PE is running*/
	private String machineName;

	/** the configuration information for the LPs that reside on this PE*/
	public final List<LPConfigInfo> lpcVector = new ArrayList<LPConfigInfo>();

	/**Constructs a new PEInfor object.
	  *@param pid int the system-assigned PE id.
	  *@param pip a String object representing the IP address  of this PE.
	  *@param pm a String object representing the name of the machine where
	  *this PE us running.
	  *@param peerStub the RMI reference for this PE.
	  */
	public PEInfo(int pid, String pip, String pm, Peer peerStub)
	{
		rmiPeerStub = peerStub;
		PEid = pid;
		IP = pip;
		machineName = pm;
	}
	
	/** Constructs a blank PE.*/
	public PEInfo()
	{
	}

	/**Returns a string representation of this object*/
	@Override
	public String toString()
	{
		String s = PEid + " " + IP + " " + machineName + " ";
		return s;
	}

	public void setPEid(int id)
	{
		this.PEid = id;
	}

	/**Sets the ip address*/
	public void setIP(String pip)
	{
		IP = pip;
	}

	/**Sets the RMI peer stub*/
	public void setPeerStub(Peer pstub)
	{
		rmiPeerStub = pstub;
	}

	/**Sets the machine name*/
	public void setMachineName(String mn)
	{
		machineName = mn;
	}

	public Peer getRmiPeerStub()
	{
		return rmiPeerStub;
	}

	public int getPEid()
	{
		return PEid;
	}

	public String getIP()
	{
		return IP;
	}

	public String getMachineName()
	{
		return machineName;
	}
} //end class
