package sassy;

import java.util.*;

/**
 * This class provides an interface for the remote method of PEs.
 *
 * @author Yin Xiong
 * @author George Vulov <georgevulov@hotmail.com>
 */

public interface Peer extends java.rmi.Remote
{
	/**
	 * Receives a message from a remote PE. Whether the receiving
	 * PE is in the process of GVT calculation.
	 * (We're using Samadi's algorithm for GVT calculation)
	 *
	 * @param msg a Message object that is to be sent to this PE.
	 * @returns true if the receiver has already reported its local min for GVT calculation
	 */
	public boolean receiveMsg(Message msg) throws java.rmi.RemoteException;

	/**
	 * <p>Informs the PE that GVT calculation has begun. The PE should
	 * calculate the local minimum of all unprocessed and unacknowledged
	 * events it holds and return it.</p>
	 *
	 * <p>After this method is called, a PE should only send marked acknowledgements
	 * when receiving messages.</p>
	 *
	 * @returns the local minimum time stamp of the PE
	 */
	public int GVTCalculationStarted() throws java.rmi.RemoteException;

	/**
	 * Informs the PE that GVT calculation has ended and passes it the new GVT value.
	 * The PE can now continue to send regular acknowledgements when receiving messages
	 * and can perform fossil collection based on the new GVT value.
	 *
	 * @param gvt the newly calculated global virtual time
	 */
	public void GVTCalculationEnded(int gvt) throws java.rmi.RemoteException;

	/**
	 * Ends the simulation running on this PE.
	 */
	public void shutDown() throws java.rmi.RemoteException;

	/**
	 * Begins the simulation running on this PE.
	 */
	public void beginSim() throws java.rmi.RemoteException;


	/**
	 * Sends the statistics information to GNS.
	 * @param lid int the id of the LP.
	  */
	public LPStat getLPStat(int lid) throws java.rmi.RemoteException;
}

