package sassy;

import java.io.*;

/**
 * This class represents the routing information for a destination LP,
 * including PE id, LP id, application id, etc.
 *
 * @author Yin Xiong
 * @author George Vulov
 * @version 2.0(10/2006)
 * @since 1.1
 */
public class RoutingInfo implements Serializable
{
	/**
	 * The RMI reference used to access the PE on which the LP resides.
	 */
	private transient Peer rmiPeerStub;

	/**The id of the PE this LP resides on*/
	private int PEid;

	/**The ip address of the PE*/
	private String PEip;

	/**The name of the machine where the PE is running*/
	private String machineName;

	/**The id of the LP*/
	private int LPid;

	/**The application id*/
	private String APPid;

	public static final long serialVersionUID = 1234L;


	public RoutingInfo(PEInfo peInfo, int lid, String appid)
	{
		PEid = peInfo.getPEid();
		PEip = peInfo.getIP();
		machineName = peInfo.getMachineName();
		rmiPeerStub = peInfo.getRmiPeerStub();
		LPid = lid;
		APPid = appid;
	}


	@Override
	public String toString()
	{
		String s = "rinfo: " + PEid + " " + PEip + " " + machineName + " " + LPid + " " + APPid;
		return s;
	} 



	/**Gets the PE machine name*/
	public String getMachineName()
	{
		return machineName;
	}


	/**Sets the PE machine name*/
	void setMachineName(String pm)
	{
		machineName = pm;
	}


	/**Sets the PEid */
	void setPEid(int pid)
	{
		PEid = pid;
	} 



	/**Sets the LPid*/
	void setLPid(int lid)
	{
		LPid = lid;
	} 



	/**Sets the APPid*/
	void setAPPid(String aid)
	{
		APPid = aid;
	}



	/**Gets the PEid of the LP*/
	public int getPEid()
	{
		return PEid;
	}



	/**Gets the LPid of the LP*/
	public int getLPid()
	{
		return LPid;
	}



	/**Gets the APPid of the LP*/
	public String getAPPid()
	{
		return APPid;
	}


	public Peer getRmiPeerStub()
	{
		return rmiPeerStub;
	}
} //end class RoutingInfo
