package sassy.agent;

import java.io.Serializable;

/**
 * These messages are sent from one agent to another.
 *
 * @author Cole Sherer <koleslaw54@gmail.com>
 */
public final class AgentMessage implements Serializable {


	/**
	 * When sending a message to another agent, the message type will be
	 * MESSAGE, if scheduling a new sense-think-act cycle, the message type
	 * will be ACT
	 */
	public static enum MessageType implements Serializable
	{
		ACT,
		SUBSCRIBE,
		UNSUBSCRIBE,
		UPDATE,
		ENTER,
		LEAVE,
		MESSAGE
	}

	private static final long serialVersionUID = -5675053696612807902L;
	private MessageType type;
	private Serializable content;


	/**
	 * Creates a new AgentMessage with the given type and content.
	 *
	 * @param type the message's type
	 * @param content the actual message to send
	 */
	public AgentMessage(MessageType type, Serializable content)
	{
		this.type = type;
		this.content = content;
	}


	/**
	 * Get the type of this message
	 *
	 * @return the message's type
	 */
	public MessageType getType()
	{
		return type;
	}


	/**
	 * Get the type of this message
	 *
	 * @return the message's type
	 */
	public Serializable getContent()
	{
		return content;
	}
}