package sassy.agent;

/**
 * A <code>BodySegment</code> that represents a standard circle.  This is
 * designed to be used in 2D simulations.
 *
 * @author Cole Sherer <koleslaw54@gmail.com>
 */
public class CircleSegment extends BodySegment {

	/**
	 * The distance between the parent Body's location and the circle's center
	 */
	private double deltaX;
	private double deltaY;
	
	private double radius;


	/**
	 * Create a CircleSegment with the same location as its parent Body, and
	 * a radius of 1.0
	 */
	public CircleSegment() {
		deltaX = deltaY = 0.0;
		radius = 1.0;
	}


	/**
	 * Create a CircleSegment that is the provided distance away from its
	 * parent Body with the given radius
	 *
	 * @param x The distance between the parent Body's x-coordinate and the
	 * segment's x-coordinate
	 * @param y The distance between the parent Body's y-coordinate and the
	 * segment's y x-coordinate
	 * @param r The radius of the circle
	 */
	public CircleSegment(double x, double y, double r) {
		deltaX = x;
		deltaY = y;
		radius = r;
	}


	/**
	 * Set the distance between the parent Body's x-coordinate and the Segment's
	 * x-coordinate
	 *
	 * @param x the distance
	 */
	public void setDeltaX(double x) {
		deltaX = x;
	}


	/**
	 * Set the distance between the parent Body's y-coordinate and the Segment's
	 * y-coordinate
	 *
	 * @param y the distance
	 */
	public void setDeltaY(double y) {
		deltaY = y;
	}


	/**
	 * Set the radius of the CircleSegment
	 *
	 * @param r the radius
	 */
	public void setRadius(double r) {
		radius = r;
	}


	/**
	 * Get the distance between the parent Body's x-coordinate and the Segment's
	 * x-coordinate
	 *
	 * @return the distance
	 */
	public double getDeltaX() {
		return deltaX;
	}


	/**
	 * Get the distance between the parent Body's y-coordinate and the Segment's
	 * y-coordinate
	 *
	 * @return the distance
	 */
	public double getDeltaY() {
		return deltaY;
	}


	/**
	 * Get the radius of the CircleSegment
	 *
	 * @return the radius
	 */
	public double getRadius() {
		return radius;
	}
}
