package sassy.agent;

/**
 * A <code>BodySegment</code> that represents a standard rectangle.  This is
 * designed to be used in 2D simulations.
 *
 * @author Cole Sherer <koleslaw54@gmail.com>
 */
public class RectangleSegment extends BodySegment {

	/**
	 * The distance between the parent Body's location and the rectangle's center
	 */
	private double deltaX;
	private double deltaY;

	private double width;
	private double height;

	private double angle;


	/**
	 * Create a RectangleSegment with the same location as its parent Body, and
	 * dimensions of 1.0 x 1.0
	 */
	public RectangleSegment() {
		deltaX = deltaY = 0.0;
		width = height = 1.0;
		angle = 0.0;
	}


	/**
	 * Create a RectangleSegment that is the provided distance away from its
	 * parent Body with the given dimensions
	 *
	 * @param x The distance between the parent Body's x-coordinate and the
	 * segment's x-coordinate
	 * @param y The distance between the parent Body's y-coordinate and the
	 * segment's y x-coordinate
	 * @param w The width of the rectangle
	 * @param h The height of the rectangle
	 * @param a The rotation of the rectangle
	 */
	public RectangleSegment(double x, double y, double w, double h, double a) {
		deltaX = x;
		deltaY = y;
		width = w;
		height = h;
		angle = a;
	}


	/**
	 * Set the distance between the parent Body's x-coordinate and the Segment's
	 * x-coordinate
	 *
	 * @param x the distance
	 */
	public void setDeltaX(double x) {
		deltaX = x;
	}


	/**
	 * Set the distance between the parent Body's y-coordinate and the Segment's
	 * y-coordinate
	 *
	 * @param y the distance
	 */
	public void setDeltaY(double y) {
		deltaY = y;
	}


	/**
	 * Set the width of the RectangleSegment
	 *
	 * @param w the width
	 */
	public void setWidth(double w) {
		width = w;
	}


	/**
	 * Set the height of the RectangleSegment
	 *
	 * @param h the height
	 */
	public void setHeight(double h) {
		height = h;
	}


	/**
	 * Set the rotation of the RectangleSegment
	 *
	 * @param a the angle of rotation
	 */
	public void setRotation(double a) {
		angle = a;
	}


	/**
	 * Get the distance between the parent Body's x-coordinate and the Segment's
	 * x-coordinate
	 *
	 * @return the distance
	 */
	public double getDeltaX() {
		return deltaX;
	}


	/**
	 * Get the distance between the parent Body's y-coordinate and the Segment's
	 * y-coordinate
	 *
	 * @return the distance
	 */
	public double getDeltaY() {
		return deltaY;
	}


	/**
	 * Get the width of the RectangleSegment
	 *
	 * @return the width
	 */
	public double getWidth() {
		return width;
	}


	/**
	 * Get the height of the RectangleSegment
	 *
	 * @return the height
	 */
	public double getHeight() {
		return height;
	}


	/**
	 * Get the rotation of the RectangleSegment
	 *
	 * @return the rotation
	 */
	public double getRotation() {
		return angle;
	}
}