package sassy.agent;

/**
 * A <code>BodySegment</code> that represents a standard sphere.  This segment
 * is designed for use in 3D simulations.
 *
 * @author Cole Sherer <koleslaw54@gmail.com>
 */
public class SphereSegment extends BodySegment {

	/**
	 * The distance from the Body co-ordinates to the center of the sphere
	 */
	private double deltaX;
	private double deltaY;
	private double deltaZ;
	
	private double radius;


	/**
	 * Creates a SphereSegment that is located at the same (x,y,z) location as
	 * the body it is attached to with a radius of 1.0
	 */
	public SphereSegment() {
		deltaX = deltaY = deltaZ = 0.0;
		radius = 1.0;
	}


	/**
	 * Creates a SphereSegment the indicated distance from the body it is
	 * attached to, with the provided radius
	 *
	 * @param x The distance between the parent body's x and the segment's x
	 * @param y The distance between the parent body's y and the segment's y
	 * @param z The distance between the parent body's z and the segment's z
	 * @param r The radius of the SphereSegment
	 */
	public SphereSegment(double x, double y, double z, double r) {
		deltaX = x;
		deltaY = y;
		deltaZ = z;
		radius = r;
	}


	/**
	 * Set the distance between the parent Body's x-coordinate and the Segment's
	 * x-coordinate
	 *
	 * @param x the distance
	 */
	public void setDeltaX(double x) {
		deltaX = x;
	}


	/**
	 * Set the distance between the parent Body's y-coordinate and the Segment's
	 * y-coordinate
	 *
	 * @param y the distance
	 */
	public void setDeltaY(double y) {
		deltaY = y;
	}


	/**
	 * Set the distance between the parent Body's z-coordinate and the Segment's
	 * z-coordinate
	 *
	 * @param z the distance
	 */
	public void setDeltaZ(double z) {
		deltaZ = z;
	}


	/**
	 * Set the radius of the SphereSegment
	 *
	 * @param r the radius
	 */
	public void setRadius(double r) {
		radius = r;
	}


	/**
	 * Get the distance between the parent Body's x-coordinate and the Segment's
	 * x-coordinate
	 *
	 * @return the distance
	 */
	public double getDeltaX() {
		return deltaX;
	}


    /**
	 * Get the distance between the parent Body's y-coordinate and the Segment's
	 * y-coordinate
	 *
	 * @return the distance
	 */
	public double getDeltaY() {
		return deltaY;
	}


	/**
	 * Get the distance between the parent Body's z-coordinate and the Segment's
	 * z-coordinate
	 *
	 * @return the distance
	 */
	public double getDeltaZ() {
		return deltaZ;
	}


	/**
	 * Get the radius of the SphereSegment
	 *
	 * @return the radius
	 */
	public double getRadius() {
		return radius;
	}
}
